@@ -3,14 +3,14 @@ import datetime
 import logging
 import os
 import random
+import re
 import socket
 import subprocess
 import sys
 import time
 import types
-import typing
 import warnings
-from typing import Dict
+from typing import Dict, Union, List, Optional
 
 import cloudpickle
 import fabric
@@ -31,13 +31,14 @@ try:
     import torch
 except ImportError:
     TORCH_ENABLED = False
+    torch = None
 else:
     TORCH_ENABLED = True
 
 logging.basicConfig(level=logging.INFO)
 
 wandb_escape = "^"
-hyperparams = None
+hyperparams: Optional[Dict] = None
 tb = tensorboard = None
 if os.path.exists("buddy_scripts/"):
     SCRIPTS_PATH = "buddy_scripts/"
@@ -73,7 +74,12 @@ def register_defaults(config_params):
                 k = k.replace("_", "-")
                 parser.add_argument(f"--{k}", f"--^{k}", type=type(v), default=v)
 
-    parsed = parser.parse_args()
+    try:
+        parsed = parser.parse_args()
+    except TypeError as e:
+        print("Type mismatch between registered hyperparameters defaults and actual values,"
+              " it might be a float argument that was passed as an int (e.g. lr=1 rather than lr=1.0) but set as a float (--lr 0.1)")
+        raise e
 
     for k, v in vars(parsed).items():
         k = k.lstrip(wandb_escape)
@@ -133,7 +139,7 @@ class WandbWrapper:
         for k, v in hyperparams.items():
             register_param(k, v)
 
-    def add_scalar(self, tag: str, scalar_value: float, global_step: int):
+    def add_scalar(self, tag: str, scalar_value: float, global_step: Optional[int] = None):
         if scalar_value != scalar_value:
             warnings.warn(f"{tag} is {scalar_value} at {global_step} :(")
 
@@ -142,11 +148,11 @@ class WandbWrapper:
         if self.tensorboard:
             self.tensorboard.add_scalar(tag, scalar_value, global_step=global_step)
 
-    def add_scalars(self, dict_of_scalars: Dict[str, float], global_step: int, prefix: str = ""):
+    def add_scalars(self, dict_of_scalars: Dict[str, float], global_step: int = None, prefix: str = ""):
         for k, v in dict_of_scalars.items():
             self.add_scalar(prefix + k, v, global_step)
 
-    def add_figure(self, tag, figure, global_step, close=True):
+    def add_figure(self, tag, figure, global_step=None, close=True):
         self.run.log({tag: figure}, global_step)
         if close:
             plt.close(figure)
@@ -155,7 +161,7 @@ class WandbWrapper:
             self.tensorboard.add_figure(tag, figure, global_step=None, close=True)
 
     @staticmethod
-    def add_histogram(tag, values, global_step):
+    def add_histogram(tag, values, global_step=None):
         if len(values) <= 2:
             raise ValueError("histogram requires at least 3 values")
 
@@ -164,11 +170,11 @@ class WandbWrapper:
         else:
             wandb.log({tag: wandb.Histogram(values)}, step=global_step, commit=False)
 
-    def plot(self, tag, values, global_step):
+    def plot(self, tag, values, global_step=None):
         wandb.log({tag: wandb.Image(values)}, step=global_step, commit=False)
         plt.close()
 
-    def add_object(self, tag, obj, global_step):
+    def add_object(self, tag, obj, global_step=None):
         if not TORCH_ENABLED:
             raise NotImplementedError
 
@@ -188,9 +194,31 @@ class WandbWrapper:
     def close(self):
         pass
 
+    def record(self, tag, value, global_step=None, exclude=None):
+        # Let wandb figure it out.
+        self.run.log({tag: value}, step=global_step, commit=False)
 
-def deploy(host: str = "", sweep_yaml: str = "", proc_num: int = 1, wandb_kwargs=None,
-           extra_slurm_headers="", extra_modules=None, disabled=False, interactive=False, run_per_agent=None) -> WandbWrapper:
+    def dump(self, step=None):
+        self.run.log({}, step=step, commit=True)
+
+
+def deploy(host: str = "", sweep_definition: Union[str, tuple] = "", proc_num: int = 1, wandb_kwargs=None,
+           extra_slurm_headers="", extra_modules=None, disabled=False, interactive=False, run_per_agent=None,
+           wandb_run_name=None
+           ) -> WandbWrapper:
+    """
+    :param host: The host to deploy to.
+    :param sweep_definition: Either a yaml file or a string containing the sweep id to resume from
+    :param proc_num: The number of parallel jobs to run.
+    :param wandb_kwargs: Kwargs to pass to wandb.init
+    :param extra_slurm_headers: Extra slurm headers to add to the job script
+    :param extra_modules: Extra modules to module load
+    :param disabled: If true does not run jobs in the cluster and invokes wandb.init with disabled=True.
+    :param interactive: Not yet implemented.
+    :param run_per_agent: If set to a number, each agent will run `run_per_agent` experiments and then exit.
+    :param wandb_run_name: If set, will use this name for the wandb run.
+    :return: A tensorboard-like object that can be used to log data.
+    """
     if wandb_kwargs is None:
         wandb_kwargs = {}
     if extra_modules is None:
@@ -204,7 +232,7 @@ def deploy(host: str = "", sweep_yaml: str = "", proc_num: int = 1, wandb_kwargs
         raise NotImplementedError("Not implemented yet")
 
     extra_slurm_headers = extra_slurm_headers.strip()
-    debug = '_pydev_bundle.pydev_log' in sys.modules.keys() and not os.environ.get('BUDDY_DEBUG_DEPLOYMENT', False)
+    debug = sys.gettrace() is not None and not os.environ.get('BUDDY_DEBUG_DEPLOYMENT', False)
     running_on_cluster = "SLURM_JOB_ID" in os.environ.keys() or "BUDDY_IS_DEPLOYED" in os.environ.keys()
     local_run = not host and not running_on_cluster
 
@@ -215,18 +243,19 @@ def deploy(host: str = "", sweep_yaml: str = "", proc_num: int = 1, wandb_kwargs
 
     project_name = experiment_buddy.utils.get_project_name(git_repo)
 
-    if local_run and sweep_yaml:
+    if local_run and sweep_definition:
         raise NotImplementedError(
             "Local sweeps are not supported.\n"
             f"SLURM_JOB_ID is {os.environ.get('SLURM_JOB_ID', 'KeyError')}\n"
             f"BUDDY_IS_DEPLOYED is {os.environ.get('BUDDY_IS_DEPLOYED', 'KeyError')}\n"
         )
-    if (local_run or sweep_yaml) and interactive:
+    if (local_run or sweep_definition) and interactive:
         raise NotImplementedError("Remote debugging requires a remote host and no sweep")
 
     wandb_kwargs = {'project': project_name, **wandb_kwargs}
     common_kwargs = {'debug': debug, 'wandb_kwargs': wandb_kwargs, }
     dtm = datetime.datetime.now().strftime("%b%d_%H-%M-%S")
+    logger = None
 
     if disabled:
         tb_dir = os.path.join(git_repo.working_dir, ARTIFACTS_PATH, "tensorboard", "DISABLED", dtm)
@@ -244,20 +273,23 @@ def deploy(host: str = "", sweep_yaml: str = "", proc_num: int = 1, wandb_kwargs
             jid = datetime.datetime.now().strftime("%b%d_%H-%M-%S")
             jid += os.environ.get("SLURM_JOB_ID", "")
             # TODO: turn into a big switch based on scheduler
-            logger = WandbWrapper(f"{experiment_id}_{jid}", **common_kwargs)
+            logger = WandbWrapper(f"{experiment_id}_{jid}", local_tensorboard=None, **common_kwargs)
     elif debug and not interactive:
         experiment_id = "DEBUG_RUN"
         tb_dir = os.path.join(git_repo.working_dir, ARTIFACTS_PATH, "tensorboard/", experiment_id, dtm)
         logger = WandbWrapper(f"{experiment_id}_{dtm}", local_tensorboard=_setup_tb(logdir=tb_dir), **common_kwargs)
     else:
         ensure_torch_compatibility()
-        experiment_id = _ask_experiment_id(host, sweep_yaml)
+        if sweep_definition and isinstance(sweep_definition, tuple):
+            experiment_id = "None, rerunning old sweep"
+        else:
+            experiment_id = wandb_run_name if wandb_run_name is not None else _ask_experiment_id(host, sweep_definition)
         print(f"experiment_id: {experiment_id}")
         if local_run:
             tb_dir = os.path.join(git_repo.working_dir, ARTIFACTS_PATH, "tensorboard/", experiment_id, dtm)
             return WandbWrapper(f"{experiment_id}_{dtm}", local_tensorboard=_setup_tb(logdir=tb_dir), **common_kwargs)
         else:
-            _commit_and_sendjob(host, experiment_id, sweep_yaml, git_repo, project_name, proc_num, extra_slurm_headers,
+            _commit_and_sendjob(host, experiment_id, sweep_definition, git_repo, project_name, proc_num, extra_slurm_headers,
                                 wandb_kwargs, extra_modules, interactive, run_per_agent)
             sys.exit()
 
@@ -271,7 +303,8 @@ def ensure_torch_compatibility():
     with open("requirements.txt") as fin:
         reqs = fin.read()
         # torch, vision or audio.
-        if "torch" not in reqs and "torch==1.7.1+cu110" not in reqs:
+        matches = re.search(r"torch==.*cu.*", reqs)
+        if "torch" in reqs and not matches:
             # https://mila-umontreal.slack.com/archives/CFAS8455H/p1624292393273100?thread_ts=1624290747.269100&cid=CFAS8455H
             warnings.warn("""torch rocm4.2 version will be installed on the cluster which is not supported specify torch==1.7.1+cu110 in requirements.txt instead""")
 
@@ -325,7 +358,7 @@ def _ensure_scripts_directory(ssh_session: fabric.Connection, extra_slurm_header
     remote_tmp_folder = retr.stdout.strip() + "/"
     ssh_session.put(f'{SCRIPTS_PATH}/common/common.sh', remote_tmp_folder)
 
-    scripts_dir = os.path.join(SCRIPTS_PATH, experiment_buddy.utils.get_backend(ssh_session, working_dir).value)
+    scripts_dir = os.path.join(SCRIPTS_PATH, experiment_buddy.utils.get_backend(ssh_session, working_dir))
 
     for file in os.listdir(scripts_dir):
         if extra_slurm_header and file in ("run_sweep.sh", "srun_python.sh"):
@@ -363,8 +396,8 @@ def log_cmd(cmd, retr):
     print("################################################################")
 
 
-def _commit_and_sendjob(hostname: str, experiment_id: str, sweep_yaml: str, git_repo: git.Repo, project_name: str,
-                        proc_num: int, extra_slurm_header: str, wandb_kwargs: dict, extra_modules=typing.List[str], interactive=False, count_per_agent=None):
+def _commit_and_sendjob(hostname: str, experiment_id: str, sweep_definition: str, git_repo: git.Repo, project_name: str,
+                        proc_num: int, extra_slurm_header: str, wandb_kwargs: dict, extra_modules=List[str], interactive=False, count_per_agent=None):
     entrypoint, extra_modules, git_url, hash_commit, scripts_folder, ssh_session = commit(
         experiment_id, extra_modules, extra_slurm_header, git_repo, hostname)
 
@@ -373,7 +406,7 @@ def _commit_and_sendjob(hostname: str, experiment_id: str, sweep_yaml: str, git_
                              project_name, scripts_folder, ssh_session, wandb_kwargs)
     else:
         send_jobs(entrypoint, experiment_id, extra_modules, git_url, hash_commit, extra_slurm_header,
-                  proc_num, project_name, scripts_folder, ssh_session, sweep_yaml, wandb_kwargs, count_per_agent)
+                  proc_num, project_name, scripts_folder, ssh_session, sweep_definition, wandb_kwargs, count_per_agent)
 
 
 def allocate_interactive(entrypoint, experiment_id, extra_modules, git_url, hash_commit,
@@ -401,10 +434,12 @@ def allocate_interactive(entrypoint, experiment_id, extra_modules, git_url, hash
 
 def send_jobs(
         entrypoint, experiment_id, extra_modules, git_url, hash_commit, extra_slurm_header,
-        proc_num, project_name, scripts_folder, ssh_session, sweep_yaml, wandb_kwargs, count_per_agent):
-
-    if sweep_yaml:
-        sweep_id = _load_sweep(entrypoint, experiment_id, project_name, sweep_yaml, wandb_kwargs)
+        proc_num, project_name, scripts_folder, ssh_session, sweep_definition, wandb_kwargs, count_per_agent):
+    if sweep_definition:
+        if isinstance(sweep_definition, str):
+            sweep_id = _load_sweep(entrypoint, experiment_id, project_name, sweep_definition, wandb_kwargs)
+        else:
+            sweep_id, hash_commit = sweep_definition  # TODO: this branch should query wandb for the old parameters {git_url}, {hash_commit} and possibly {extra_modules}
         ssh_command = f"/opt/slurm/bin/sbatch {scripts_folder}/run_sweep.sh {git_url} {sweep_id} {hash_commit} {extra_modules} {count_per_agent}"
     else:
         ssh_command = f"bash -l {scripts_folder}/run_experiment.sh {git_url} {entrypoint} {hash_commit} {extra_modules}"
