@@ -36,8 +36,13 @@ function load_git_folder() {
   mkdir -p "$HOME"/experiments/
   cd "$HOME"/experiments/
 
-  EXPERIMENT_FOLDER=$(mktemp -p . -d)
+  EXPERIMENT_FOLDER="$(mktemp -p . -d)"
+
   pull_experiment $GIT_URL $HASH_COMMIT $EXPERIMENT_FOLDER
+  EXPERIMENT_NAME=$(git log --format=%B -n 1)
+  cd ".."
+  mv "$EXPERIMENT_FOLDER" "$EXPERIMENT_FOLDER-$EXPERIMENT_NAME"
+  cd "$EXPERIMENT_FOLDER-$EXPERIMENT_NAME"
 }
 
 log "running on $(hostname)"
@@ -68,11 +73,14 @@ log "Using shared venv @ $HOME/venv"
 log "Upgrading pip"
 python3 -m pip install --upgrade pip
 log "Upgrading requirements"
-python3 -m pip install --upgrade -r "requirements.txt" --exists-action w -f https://download.pytorch.org/whl/torch_stable.html -f https://storage.googleapis.com/jax-releases/jax_releases.html | grep -v "Requirement already satisfied"
+
+# cat requirements.txt | xargs --max-args=1 --max-procs=20 python3 -m pip install --exists-action s -f https://download.pytorch.org/whl/torch_stable.html -f https://storage.googleapis.com/jax-releases/jax_releases.html | grep -v "Requirement already satisfied"
+python3 -m pip cache purge
+python3 -m pip install --upgrade -r "requirements.txt" --exists-action s -f https://download.pytorch.org/whl/torch_stable.html -f https://storage.googleapis.com/jax-releases/jax_releases.html | grep -v "Requirement already satisfied"
 log "Requirements upgraded"
 
 export XLA_FLAGS=--xla_gpu_cuda_data_dir=/cvmfs/ai.mila.quebec/apps/x86_64/common/cuda/10.1/
 
 # TODO: the client should send the mila_tools version to avoid issues
 log "/opt/slurm/bin/sbatch $SCRIPTS_FOLDER/srun_python.sh $ENTRYPOINT"
-/opt/slurm/bin/sbatch --comment "$(git log --format=%B -n 1)" $SCRIPTS_FOLDER/srun_python.sh $ENTRYPOINT
+/opt/slurm/bin/sbatch --comment "$EXPERIMENT_NAME" $SCRIPTS_FOLDER/srun_python.sh $ENTRYPOINT
